"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataarray_serialization = exports.getArrayProp = void 0;
const base_1 = require("@jupyter-widgets/base");
function getArrayProp(obj) {
    if (obj === null) {
        return null;
    }
    else if (obj instanceof base_1.WidgetModel) {
        // we assume it is a NDArrayModel
        // TODO: handle Promise?
        const ndArray = obj.getNDArray();
        if (ndArray.shape.length == 1) {
            return new Float32Array(ndArray.data);
        }
        else {
            const arrays = [];
            return arrays;
        }
    }
    else {
        return obj;
    }
}
exports.getArrayProp = getArrayProp;
function concat(arrays, size) {
    const array = new Float32Array(arrays.length * size);
    let offset = 0;
    for (const arr of arrays) {
        array.set(arr, offset);
        offset += size;
    }
    return array;
}
function split(array, shape) {
    const arrays = [];
    let offset = 0;
    const size = shape[1];
    const end = size;
    for (let i = 0; i < shape[0]; i++) {
        arrays.push(array.subarray(offset, end));
        offset = end;
    }
    return arrays;
}
function deserializeArray(value, manager) {
    const arr = new Float32Array(value.buffer.buffer);
    if (value.shape.length == 1) {
        return arr;
    }
    else {
        return split(arr, value.shape);
    }
}
function deserializeDataArray(value, manager) {
    if (value === null) {
        return null;
    }
    else if (typeof value === 'string') {
        return (0, base_1.unpack_models)(value, manager);
    }
    else {
        return deserializeArray(value, manager);
    }
}
function serializeDataArray(obj, _widget) {
    if (obj === null) {
        return null;
    }
    else if (obj instanceof base_1.WidgetModel) {
        return 'IPY_MODEL_' + obj.model_id;
    }
    else if (obj instanceof Float32Array) {
        return { shape: obj.length, dtype: 'float32', buffer: obj };
    }
    else {
        const arrays = obj;
        const size = arrays[0].length;
        const array = concat(arrays, size);
        const shape = [arrays.length, size];
        return { shape: shape, dtype: 'float32', buffer: array };
    }
}
exports.dataarray_serialization = {
    deserialize: deserializeDataArray,
    serialize: serializeDataArray,
};
//# sourceMappingURL=serializers.js.map