"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FFTModel = exports.WaveformModel = exports.DCMeterModel = exports.MeterModel = exports.AnalyserModel = void 0;
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
class AnalyserModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AnalyserModel.model_name, _channels: 1, type: 'fft', size: 1024, smoothing: 0.8 });
    }
    createNode() {
        return new tone.Analyser({
            channels: this.get('_channels'),
            type: this.get('type'),
            size: this.get('size'),
            smoothing: this.get('smoothing'),
        });
    }
    getValueAtTime(traitName, _time) {
        return this.getValue(traitName);
    }
    getValue(traitName) {
        (0, utils_1.assert)(traitName === 'array', 'envelope only supports "array" trait');
        return this.node.getValue();
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:type', () => {
            this.node.type = this.get('type');
        });
        this.on('change:size', () => {
            this.node.size = this.get('size');
        });
        this.on('change:smoothing', () => {
            this.node.smoothing = this.get('smoothing');
        });
    }
}
exports.AnalyserModel = AnalyserModel;
AnalyserModel.model_name = 'AnalyserModel';
class BaseMeterModel extends widget_base_1.AudioNodeModel {
    getValueAtTime(traitName, _time) {
        return this.getValue(traitName);
    }
    getValue(traitName) {
        (0, utils_1.assert)(traitName === 'value' || traitName === 'array', 'Meters only supports "value" or "array" trait');
        let value = this.node.getValue();
        // Issue with -Infinity (inactive signal in dB) converted to None in Python
        // https://github.com/jupyter-widgets/ipywidgets/issues/3321
        // https://github.com/jupyter-widgets/ipywidgets/issues/1531
        if (value === -Infinity) {
            value = -1e20;
        }
        else if (Array.isArray(value) && value.length == 2) {
            value[0] = value[0] === -Infinity ? -1e20 : value[0];
            value[1] = value[1] === -Infinity ? -1e20 : value[1];
        }
        return value;
    }
}
class MeterModel extends BaseMeterModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: MeterModel.model_name, _channels: 1, normal_range: false, smoothing: 0.8 });
    }
    createNode() {
        return new tone.Meter({
            channels: this.get('_channels'),
            normalRange: this.get('normal_range'),
            smoothing: this.get('smoothing'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:normal_range', () => {
            this.node.normalRange = this.get('normal_range');
        });
        this.on('change:smoothing', () => {
            this.node.smoothing = this.get('smoothing');
        });
    }
}
exports.MeterModel = MeterModel;
MeterModel.model_name = 'MeterModel';
class DCMeterModel extends BaseMeterModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: DCMeterModel.model_name });
    }
    createNode() {
        return new tone.DCMeter();
    }
}
exports.DCMeterModel = DCMeterModel;
DCMeterModel.model_name = 'DCMeterModel';
class WaveformModel extends BaseMeterModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: WaveformModel.model_name, size: 1024 });
    }
    createNode() {
        return new tone.Waveform({
            size: this.get('size'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:size', () => {
            this.node.size = this.get('size');
        });
    }
}
exports.WaveformModel = WaveformModel;
WaveformModel.model_name = 'WaveformModel';
class FFTModel extends BaseMeterModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: FFTModel.model_name, size: 1024, normal_range: false, smoothing: 0.8 });
    }
    createNode() {
        return new tone.FFT({
            size: this.get('size'),
            normalRange: this.get('normal_range'),
            smoothing: this.get('smoothing'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:size', () => {
            this.node.size = this.get('size');
        });
        this.on('change:normal_range', () => {
            this.node.normalRange = this.get('normal_range');
        });
        this.on('change:smoothing', () => {
            this.node.smoothing = this.get('smoothing');
        });
    }
}
exports.FFTModel = FFTModel;
FFTModel.model_name = 'FFTModel';
//# sourceMappingURL=widget_analysis.js.map