/// <reference types="backbone" />
import { WidgetModel, ISerializers } from '@jupyter-widgets/base';
import * as tone from 'tone';
export declare abstract class ToneWidgetModel extends WidgetModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    initEventListeners(): void;
    static model_module: any;
    static model_module_version: any;
}
export declare class NativeAudioNodeModel extends ToneWidgetModel {
    defaults(): any;
    connectInputCallback(): void;
    setNode(node: AudioNode): void;
    node: AudioNode;
    static model_name: string;
}
export declare class NativeAudioParamModel extends ToneWidgetModel {
    defaults(): any;
    connectInputCallback(): void;
    setNode(node: AudioParam): void;
    node: AudioParam;
    static model_name: string;
}
export declare abstract class ToneObjectModel extends ToneWidgetModel {
    defaults(): any;
    setNode(node: any): void;
    get disposed(): boolean;
    dispose(): void;
    initEventListeners(): void;
    node: any;
    static model_name: string;
}
export declare class NodeWithContextModel extends ToneObjectModel {
    defaults(): any;
    static model_name: string;
}
export declare abstract class AudioNodeModel extends NodeWithContextModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    get input(): ToneObjectModel | NativeAudioNodeModel | NativeAudioParamModel;
    get output(): ToneObjectModel | NativeAudioNodeModel;
    connectInputCallback(): void;
    node: tone.ToneAudioNode;
    abstract createNode(): tone.ToneAudioNode;
    setNode(node: tone.ToneAudioNode): void;
    setNodeChannels(): void;
    private setInputOutputNodes;
    setSubNodes(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    static model_name: string;
}
/**
 * A basic Tone.js audio node that just accepts any input/output.
 * It is meant to be wrapped by a PyInternalAudioNode widget.
 */
export declare class PyToneAudioNode extends tone.ToneAudioNode {
    readonly name: string;
    readonly input: tone.InputNode | undefined;
    readonly output: tone.OutputNode | undefined;
    constructor(input?: tone.InputNode, output?: tone.OutputNode);
}
export declare class PyInternalAudioNodeModel extends AudioNodeModel {
    defaults(): any;
    createNode(): PyToneAudioNode;
    node: PyToneAudioNode;
    static model_name: string;
}
