/// <reference types="backbone" />
import { ISerializers } from '@jupyter-widgets/base';
import * as tone from 'tone';
import { UnitMap, UnitName } from 'tone/Tone/core/type/Units';
import { ObservableModel } from './widget_observe';
import { AudioNodeModel, NativeAudioNodeModel, NativeAudioParamModel, NodeWithContextModel, ToneObjectModel } from './widget_base';
import { ArrayProperty } from './serializers';
export declare class InternalAudioNodeModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.ToneAudioNode;
    static model_name: string;
}
interface ParamProperties<T extends UnitName> {
    value: UnitMap[T];
    units: T;
    convert: boolean;
    minValue?: number;
    maxValue?: number;
    gain?: UnitMap[T];
}
export declare class ParamModel<T extends UnitName> extends NodeWithContextModel implements ObservableModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    get input(): NativeAudioParamModel | NativeAudioNodeModel;
    get value(): UnitMap[T];
    set value(val: UnitMap[T]);
    get units(): T;
    get convert(): boolean;
    get minValue(): number | undefined;
    get maxValue(): number | undefined;
    get properties(): ParamProperties<T>;
    getValueAtTime(traitName: string, time: tone.Unit.Seconds): UnitMap[T];
    getValue(traitName: string): UnitMap[T];
    private normalizeMinMax;
    setNode(node: tone.Param<T>): void;
    connectInputCallback(): void;
    dispose(): void;
    private handleMsg;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.Param<T>;
    static model_name: string;
}
export declare class GainModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.Gain;
    setSubNodes(): void;
    get gain(): ParamModel<'gain'>;
    static serializers: ISerializers;
    node: tone.Gain;
    static model_name: string;
}
export declare class VolumeModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.Volume;
    get volume(): ParamModel<'decibels'>;
    get mute(): boolean;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.Volume;
    static model_name: string;
}
export declare class DestinationModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.ToneAudioNode;
    get mute(): boolean;
    get volume(): ParamModel<'decibels'>;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: typeof tone.Destination;
    static model_name: string;
}
export declare class AudioBufferModel extends ToneObjectModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    get buffer_url(): null | string;
    get array(): ArrayProperty;
    set array(value: ArrayProperty);
    fromArray(array: Float32Array | Float32Array[]): void;
    fromUrl(url: string): void;
    setNode(node: tone.ToneAudioBuffer): void;
    resetBufferProperties(): void;
    setBufferProperties(): void;
    initEventListeners(): void;
    node: tone.ToneAudioBuffer;
    static serializers: ISerializers;
    static model_name: string;
}
export declare class AudioBuffersModel extends ToneObjectModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    get buffers(): Map<string, AudioBufferModel>;
    get buffer_nodes(): tone.ToneAudioBuffersUrlMap;
    setNode(node: tone.ToneAudioBuffers): void;
    private updateBufferNodes;
    initEventListeners(): void;
    node: tone.ToneAudioBuffers;
    static serializers: ISerializers;
    static model_name: string;
}
export declare class ListenerModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.ToneAudioNode;
    setSubNodes(): void;
    static serializers: ISerializers;
    node: typeof tone.Listener;
    static model_name: string;
}
export {};
