"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListenerModel = exports.AudioBuffersModel = exports.AudioBufferModel = exports.DestinationModel = exports.VolumeModel = exports.GainModel = exports.ParamModel = exports.InternalAudioNodeModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
const serializers_1 = require("./serializers");
class InternalAudioNodeModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: InternalAudioNodeModel.model_name, _n_inputs: 1, _n_outputs: 1, type: '', _create_node: false });
    }
    createNode() {
        throw new Error('Not implemented');
    }
}
exports.InternalAudioNodeModel = InternalAudioNodeModel;
InternalAudioNodeModel.model_name = 'InternalAudioNodeModel';
class ParamModel extends widget_base_1.NodeWithContextModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ParamModel.model_name, _create_node: true, _input: null, value: 0, convert: true, _units: 'number', _min_value: undefined, _max_value: undefined, _swappable: false, overridden: false });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        if (this.get('_create_node')) {
            // TODO: is there any use case of creating a new Tone.Param from Python?
            // Usually a ParamModel wraps an existing Tone.Param object
            this.node = new tone.Param({
                convert: this.convert,
                value: this.value,
                units: this.units,
                minValue: this.normalizeMinMax(this.get('_min_value')),
                maxValue: this.normalizeMinMax(this.get('_max_value')),
            });
            this.input.node = this.node.input;
        }
    }
    get input() {
        return this.get('_input');
    }
    get value() {
        return this.get('value');
    }
    set value(val) {
        this.set('value', val);
    }
    get units() {
        return this.get('_units');
    }
    get convert() {
        return this.get('convert');
    }
    get minValue() {
        return this.normalizeMinMax(this.get('_min_value'));
    }
    get maxValue() {
        return this.normalizeMinMax(this.get('_max_value'));
    }
    get properties() {
        const opts = {
            value: this.value,
            units: this.units,
            convert: this.convert,
        };
        if (this.minValue !== undefined) {
            opts.minValue = this.minValue;
        }
        if (this.maxValue !== undefined) {
            opts.maxValue = this.maxValue;
        }
        return opts;
    }
    getValueAtTime(traitName, time) {
        (0, utils_1.assert)(traitName === 'value', 'param only supports "value" trait');
        return this.node.getValueAtTime(time);
    }
    getValue(traitName) {
        (0, utils_1.assert)(traitName === 'value', 'param only supports "value" trait');
        return this.node.value;
    }
    normalizeMinMax(value) {
        return value === null ? undefined : value;
    }
    setNode(node) {
        this.node = node;
        this.input.setNode(node.input);
    }
    connectInputCallback() {
        this.set('overridden', this.node.overridden);
        // if overridden, value is reset to 0
        this.set('value', this.node.value);
        this.save_changes();
    }
    dispose() {
        // Tone.js may have called dispose internally for this Tone instance
        // so we need to check if the instance is already disposed
        if (!this.node.disposed) {
            this.node.dispose();
        }
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:value', () => {
            this.node.value = this.get('value');
        });
        this.on('change:convert', () => {
            this.node.convert = this.get('convert');
        });
        this.on('change:_disposed', this.dispose, this);
        this.on('msg:custom', this.handleMsg, this);
    }
}
exports.ParamModel = ParamModel;
ParamModel.serializers = Object.assign(Object.assign({}, widget_base_1.NodeWithContextModel.serializers), { _input: { deserialize: base_1.unpack_models } });
ParamModel.model_name = 'ParamModel';
class GainModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: GainModel.model_name, _gain: undefined });
    }
    createNode() {
        const gain_opts = this.gain.properties;
        gain_opts.gain = gain_opts.value;
        return new tone.Gain(gain_opts);
    }
    setSubNodes() {
        super.setSubNodes();
        this.gain.setNode(this.node.gain);
    }
    get gain() {
        return this.get('_gain');
    }
}
exports.GainModel = GainModel;
GainModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _gain: { deserialize: base_1.unpack_models } });
GainModel.model_name = 'GainModel';
class VolumeModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: VolumeModel.model_name, _volume: undefined, mute: false });
    }
    createNode() {
        return new tone.Volume({ volume: this.volume.value, mute: this.mute });
    }
    get volume() {
        return this.get('_volume');
    }
    get mute() {
        return this.get('mute');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:mute', () => {
            this.node.mute = this.mute;
            // update volume param model value
            this.volume.value = this.node.volume.value;
            this.volume.save_changes();
        });
    }
}
exports.VolumeModel = VolumeModel;
VolumeModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _volume: { deserialize: base_1.unpack_models } });
VolumeModel.model_name = 'VolumeModel';
class DestinationModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: DestinationModel.model_name, _volume: undefined, mute: false });
    }
    createNode() {
        return tone.getDestination();
    }
    get mute() {
        return this.get('mute');
    }
    get volume() {
        return this.get('_volume');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:mute', () => {
            this.node.mute = this.mute;
            // update volume param model value
            this.volume.value = this.node.volume.value;
            this.volume.save_changes();
        });
    }
}
exports.DestinationModel = DestinationModel;
DestinationModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _volume: { deserialize: base_1.unpack_models } });
DestinationModel.model_name = 'DestinationModel';
class AudioBufferModel extends widget_base_1.ToneObjectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AudioBufferModel.model_name, buffer_url: null, array: null, _sync_array: false, duration: 0, length: 0, n_channels: 0, sample_rate: 0, loaded: false, reverse: false, _create_node: true });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        if (this.get('_create_node')) {
            this.node = new tone.ToneAudioBuffer({ reverse: this.get('reverse') });
            if (this.array !== null) {
                this.fromArray(this.array);
            }
            else if (this.buffer_url !== null) {
                this.fromUrl(this.buffer_url);
            }
        }
    }
    get buffer_url() {
        return this.get('buffer_url');
    }
    get array() {
        return (0, serializers_1.getArrayProp)(this.get('array'));
    }
    set array(value) {
        // avoid infinite event listener loop
        this.set('array', value, { silent: true });
    }
    fromArray(array) {
        this.set('buffer_url', null);
        this.node.fromArray(array);
        this.setBufferProperties();
    }
    fromUrl(url) {
        this.resetBufferProperties();
        this.node.load(url).then(() => {
            this.setBufferProperties();
        });
    }
    setNode(node) {
        this.node = node;
        if (node.loaded) {
            this.setBufferProperties();
        }
        else {
            node.onload = () => {
                this.setBufferProperties();
            };
        }
    }
    resetBufferProperties() {
        this.set('duration', 0);
        this.set('length', 0);
        this.set('n_channels', 0);
        this.set('sample_rate', 0);
        this.set('loaded', false);
        this.array = null;
        this.save_changes();
    }
    setBufferProperties() {
        this.set('duration', this.node.duration);
        this.set('length', this.node.length);
        this.set('n_channels', this.node.numberOfChannels);
        this.set('sample_rate', this.node.sampleRate);
        this.set('loaded', this.node.loaded);
        this.set('reverse', this.node.reverse);
        if (this.get('_sync_array') && this.node.duration < 10) {
            this.array = this.node.toArray();
        }
        this.save_changes();
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:array', () => {
            if (this.array !== null) {
                this.fromArray(this.array);
            }
        });
        this.on('change:buffer_url', () => {
            if (this.buffer_url !== null) {
                this.fromUrl(this.buffer_url);
            }
        });
        this.on('change:reverse', () => {
            this.node.reverse = this.get('reverse');
        });
    }
}
exports.AudioBufferModel = AudioBufferModel;
AudioBufferModel.serializers = Object.assign(Object.assign({}, widget_base_1.ToneObjectModel.serializers), { array: serializers_1.dataarray_serialization });
AudioBufferModel.model_name = 'AudioBufferModel';
class AudioBuffersModel extends widget_base_1.ToneObjectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AudioBuffersModel.model_name, _base_url: '', _buffers: null, _create_node: true });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        if (this.get('_create_node')) {
            this.node = new tone.ToneAudioBuffers({
                urls: this.buffer_nodes,
                baseUrl: this.get('_base_url'),
            });
        }
    }
    get buffers() {
        return new Map(Object.entries(this.get('_buffers')));
    }
    get buffer_nodes() {
        const nodes = {};
        this.buffers.forEach((buf, key) => {
            if (buf.node === undefined) {
                nodes[key] = buf.buffer_url;
            }
            else {
                nodes[key] = buf.node;
            }
        });
        return nodes;
    }
    setNode(node) {
        this.node = node;
        this.buffers.forEach((buf, key) => {
            let bufferNode;
            if (node.has(key)) {
                bufferNode = node.get(key);
            }
            else {
                // try with key converted to Midi (case of tone.Sampler)
                const keyMidi = new tone.FrequencyClass(tone.context, key).toMidi();
                bufferNode = node.get(keyMidi);
            }
            buf.setNode(bufferNode);
        });
    }
    updateBufferNodes() {
        this.buffers.forEach((buf, key) => {
            this.node.add(key, buf.node);
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:_buffers', () => {
            this.updateBufferNodes();
        });
    }
}
exports.AudioBuffersModel = AudioBuffersModel;
AudioBuffersModel.serializers = Object.assign(Object.assign({}, widget_base_1.ToneObjectModel.serializers), { _buffers: { deserialize: base_1.unpack_models } });
AudioBuffersModel.model_name = 'AudioBuffersModel';
class ListenerModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ListenerModel.model_name, _position_x: undefined, _position_y: undefined, _position_z: undefined, _forward_x: undefined, _forward_y: undefined, _forward_z: undefined, _up_x: undefined, _up_y: undefined, _up_z: undefined });
    }
    createNode() {
        return tone.getListener();
    }
    setSubNodes() {
        super.setSubNodes();
        this.get('_position_x').setNode(this.node.positionX);
        this.get('_position_y').setNode(this.node.positionY);
        this.get('_position_z').setNode(this.node.positionZ);
        this.get('_forward_x').setNode(this.node.forwardX);
        this.get('_forward_y').setNode(this.node.forwardY);
        this.get('_forward_z').setNode(this.node.forwardZ);
        this.get('_up_x').setNode(this.node.upX);
        this.get('_up_y').setNode(this.node.upY);
        this.get('_up_z').setNode(this.node.upZ);
    }
}
exports.ListenerModel = ListenerModel;
ListenerModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _position_x: { deserialize: base_1.unpack_models }, _position_y: { deserialize: base_1.unpack_models }, _position_z: { deserialize: base_1.unpack_models }, _forward_x: { deserialize: base_1.unpack_models }, _forward_y: { deserialize: base_1.unpack_models }, _forward_z: { deserialize: base_1.unpack_models }, _up_x: { deserialize: base_1.unpack_models }, _up_y: { deserialize: base_1.unpack_models }, _up_z: { deserialize: base_1.unpack_models } });
ListenerModel.model_name = 'ListenerModel';
//# sourceMappingURL=widget_core.js.map