"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.VibratoModel = exports.TremoloModel = exports.ReverbModel = exports.PitchShiftModel = exports.PingPongDelayModel = exports.FrequencyShifterModel = exports.FeedbackDelayModel = exports.DistortionModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
class EffectModel extends widget_base_1.AudioNodeModel {
    // Returns the wet value from the output node model.
    // This allows to pass it to the constructor of the
    // Tone.Effect subclasses.
    get wetFromModel() {
        return this.output.fade.value;
    }
}
EffectModel.model_name = 'EffectModel';
class DistortionModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: DistortionModel.model_name, distortion: 0.4, oversample: 'none' });
    }
    createNode() {
        return new tone.Distortion({
            distortion: this.get('distortion'),
            oversample: this.get('oversample'),
            wet: this.wetFromModel,
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:distortion', () => {
            this.node.distortion = this.get('distortion');
        });
        this.on('change:oversample', () => {
            this.node.oversample = this.get('oversample');
        });
    }
}
exports.DistortionModel = DistortionModel;
DistortionModel.model_name = 'DistortionModel';
class FeedbackDelayModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: FeedbackDelayModel.model_name, _max_delay: 1, _delay_time: undefined, _feedback: undefined });
    }
    createNode() {
        return new tone.FeedbackDelay({
            delayTime: this.delayTime.value,
            feedback: this.feedback.value,
            maxDelay: this.get('_max_delay'),
            wet: this.wetFromModel,
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.delayTime.setNode(this.node.delayTime);
        this.feedback.setNode(this.node.feedback);
    }
    get delayTime() {
        return this.get('_delay_time');
    }
    get feedback() {
        return this.get('_feedback');
    }
}
exports.FeedbackDelayModel = FeedbackDelayModel;
FeedbackDelayModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _delay_time: { deserialize: base_1.unpack_models }, _feedback: { deserialize: base_1.unpack_models } });
FeedbackDelayModel.model_name = 'FeedbackDelayModel';
class FrequencyShifterModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: FrequencyShifterModel.model_name, _frequency: undefined });
    }
    createNode() {
        return new tone.FrequencyShifter({
            frequency: this.frequency.value,
            wet: this.wetFromModel,
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.frequency.setNode(this.node.frequency);
    }
    get frequency() {
        return this.get('_frequency');
    }
}
exports.FrequencyShifterModel = FrequencyShifterModel;
FrequencyShifterModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _frequency: { deserialize: base_1.unpack_models } });
FrequencyShifterModel.model_name = 'FrequencyShifterModel';
class PingPongDelayModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PingPongDelayModel.model_name, _max_delay: 1, _delay_time: undefined, _feedback: undefined });
    }
    createNode() {
        return new tone.PingPongDelay({
            delayTime: this.delayTime.value,
            feedback: this.feedback.value,
            maxDelay: this.get('_max_delay'),
            wet: this.wetFromModel,
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.delayTime.setNode(this.node.delayTime);
        this.feedback.setNode(this.node.feedback);
    }
    get delayTime() {
        return this.get('_delay_time');
    }
    get feedback() {
        return this.get('_feedback');
    }
}
exports.PingPongDelayModel = PingPongDelayModel;
PingPongDelayModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _delay_time: { deserialize: base_1.unpack_models }, _feedback: { deserialize: base_1.unpack_models } });
PingPongDelayModel.model_name = 'PingPongDelayModel';
class PitchShiftModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PitchShiftModel.model_name, pitch: 0, window_size: 0.1, _delay_time: undefined, _feedback: undefined });
    }
    createNode() {
        return new tone.PitchShift({
            pitch: this.get('pitch'),
            windowSize: this.get('window_size'),
            delayTime: this.delayTime.value,
            feedback: this.feedback.value,
            wet: this.wetFromModel,
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.delayTime.setNode(this.node.delayTime);
        this.feedback.setNode(this.node.feedback);
    }
    get delayTime() {
        return this.get('_delay_time');
    }
    get feedback() {
        return this.get('_feedback');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:pitch', () => {
            this.node.pitch = this.get('pitch');
        });
        this.on('change:window_size', () => {
            this.node.windowSize = this.get('window_size');
        });
    }
}
exports.PitchShiftModel = PitchShiftModel;
PitchShiftModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _delay_time: { deserialize: base_1.unpack_models }, _feedback: { deserialize: base_1.unpack_models } });
PitchShiftModel.model_name = 'PitchShiftModel';
class ReverbModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ReverbModel.model_name, decay: 1.5, pre_delay: 0.01 });
    }
    initialize(attributes, options) {
        // special case: reverb impulse response is generated asynchronuously
        if (this.get('_create_node')) {
            this.node = this.createNode();
            this.node.ready.then(() => {
                this.input.setNode(this.node.input);
                this.output.setNode(this.node.output);
            });
            this.set('_create_node', false);
            this.save_changes();
        }
        super.initialize(attributes, options);
    }
    createNode() {
        return new tone.Reverb({
            decay: this.get('decay'),
            preDelay: this.get('pre_delay'),
            wet: this.wetFromModel,
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:decay', () => {
            this.node.decay = this.get('decay');
        });
        this.on('change:pre_delay', () => {
            this.node.preDelay = this.get('pre_delay');
        });
    }
}
exports.ReverbModel = ReverbModel;
ReverbModel.model_name = 'ReverbModel';
class TremoloModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: TremoloModel.model_name, _frequency: undefined, _depth: undefined, type: 'sine', spread: 180 });
    }
    createNode() {
        return new tone.Tremolo({
            frequency: this.frequency.value,
            depth: this.depth.value,
            type: this.get('type'),
            spread: this.get('spread'),
            wet: this.wetFromModel,
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.frequency.setNode(this.node.frequency);
        this.depth.setNode(this.node.depth);
    }
    get frequency() {
        return this.get('_frequency');
    }
    get depth() {
        return this.get('_depth');
    }
    get type() {
        return this.get('type');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:spread', () => {
            this.node.spread = this.get('spread');
        });
        this.on('change:type', () => {
            this.node.type = this.type;
        });
        this.on('msg:custom', this.handleMsg, this);
    }
    handleMsg(command, buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
}
exports.TremoloModel = TremoloModel;
TremoloModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _frequency: { deserialize: base_1.unpack_models }, _depth: { deserialize: base_1.unpack_models } });
TremoloModel.model_name = 'TremoloModel';
class VibratoModel extends EffectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: VibratoModel.model_name, _max_delay: 0.005, _frequency: undefined, _depth: undefined, type: 'sine' });
    }
    createNode() {
        return new tone.Vibrato({
            frequency: this.frequency.value,
            depth: this.depth.value,
            type: this.get('type'),
            maxDelay: this.get('_max_delay'),
            wet: this.wetFromModel,
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.frequency.setNode(this.node.frequency);
        this.depth.setNode(this.node.depth);
    }
    get frequency() {
        return this.get('_frequency');
    }
    get depth() {
        return this.get('_depth');
    }
    get type() {
        return this.get('type');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:type', () => {
            this.node.type = this.type;
        });
    }
}
exports.VibratoModel = VibratoModel;
VibratoModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _frequency: { deserialize: base_1.unpack_models }, _depth: { deserialize: base_1.unpack_models } });
VibratoModel.model_name = 'VibratoModel';
//# sourceMappingURL=widget_effect.js.map