"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FrequencyEnvelopeModel = exports.AmplitudeEnvelopeModel = exports.EnvelopeModel = void 0;
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
const serializers_1 = require("./serializers");
class EnvelopeModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: EnvelopeModel.model_name, attack: 0.01, decay: 0.1, sustain: 1, release: 0.5, attack_curve: 'linear', decay_curve: 'exponential', release_curve: 'exponential', array: null, array_length: 1024, sync_array: false });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        if (this.get('_create_node')) {
            this.maybeSetArray();
        }
    }
    get array() {
        return (0, serializers_1.getArrayProp)(this.get('array'));
    }
    set array(value) {
        // avoid infinite event listener loop
        this.set('array', value, { silent: true });
    }
    createNode() {
        return new tone.Envelope({
            attack: this.get('attack'),
            decay: this.get('decay'),
            sustain: this.get('sustain'),
            release: this.get('release'),
            attackCurve: this.get('attack_curve'),
            decayCurve: this.get('decay_curve'),
            releaseCurve: this.get('release_curve'),
        });
    }
    setNode(node) {
        super.setNode(node);
        this.maybeSetArray();
    }
    maybeSetArray() {
        if (this.get('sync_array')) {
            this.node.asArray(this.get('array_length')).then((arr) => {
                this.array = arr;
                this.save_changes();
            });
        }
    }
    updateEnvelope(trait_name, node_property) {
        this.node[node_property] = this.get(trait_name);
        this.maybeSetArray();
    }
    getValueAtTime(traitName, time) {
        (0, utils_1.assert)(traitName === 'value', 'envelope only supports "value" trait');
        return this.node.getValueAtTime(time);
    }
    getValue(traitName) {
        (0, utils_1.assert)(traitName === 'value', 'envelope only supports "value" trait');
        return this.node.value;
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:attack', () => this.updateEnvelope('attack', 'attack'));
        this.on('change:decay', () => this.updateEnvelope('decay', 'decay'));
        this.on('change:sustain', () => this.updateEnvelope('sustain', 'sustain'));
        this.on('change:release', () => this.updateEnvelope('release', 'release'));
        this.on('change:attack_curve', () => this.updateEnvelope('attack_curve', 'attackCurve'));
        this.on('change:decay_curve', () => this.updateEnvelope('decay_curve', 'decayCurve'));
        this.on('change:release_curve', () => this.updateEnvelope('release_curve', 'releaseCurve'));
        this.on('msg:custom', this.handleMsg, this);
    }
}
exports.EnvelopeModel = EnvelopeModel;
EnvelopeModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { array: serializers_1.dataarray_serialization });
EnvelopeModel.model_name = 'EnvelopeModel';
class AmplitudeEnvelopeModel extends EnvelopeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AmplitudeEnvelopeModel.model_name });
    }
    createNode() {
        return new tone.AmplitudeEnvelope({
            attack: this.get('attack'),
            decay: this.get('decay'),
            sustain: this.get('sustain'),
            release: this.get('release'),
            attackCurve: this.get('attack_curve'),
            decayCurve: this.get('decay_curve'),
            releaseCurve: this.get('release_curve'),
        });
    }
}
exports.AmplitudeEnvelopeModel = AmplitudeEnvelopeModel;
AmplitudeEnvelopeModel.model_name = 'AmplitudeEnvelopeModel';
class FrequencyEnvelopeModel extends EnvelopeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: FrequencyEnvelopeModel.model_name, base_frequency: 200.0, octaves: 4, exponent: 1 });
    }
    createNode() {
        return new tone.FrequencyEnvelope({
            attack: this.get('attack'),
            decay: this.get('decay'),
            sustain: this.get('sustain'),
            release: this.get('release'),
            attackCurve: this.get('attack_curve'),
            decayCurve: this.get('decay_curve'),
            releaseCurve: this.get('release_curve'),
            baseFrequency: this.get('base_frequency'),
            octaves: this.get('octaves'),
            exponent: this.get('exponent'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:base_frequency', () => {
            this.node.baseFrequency = this.get('base_frequency');
        });
        this.on('change:octaves', () => {
            this.node.octaves = this.get('octaves');
        });
        this.on('change:exponent', () => {
            this.node.exponent = this.get('exponent');
        });
    }
}
exports.FrequencyEnvelopeModel = FrequencyEnvelopeModel;
FrequencyEnvelopeModel.model_name = 'FrequencyEnvelopeModel';
//# sourceMappingURL=widget_envelope.js.map