/// <reference types="backbone" />
import * as tone from 'tone';
import { NodeWithContextModel } from './widget_base';
import { ObservableModel } from './widget_observe';
interface Event {
    callback: tone.ToneEventCallback<any>;
    humanize: boolean | tone.Unit.Time;
    mute: boolean;
    probability: tone.Unit.NormalRange;
    playbackRate: tone.Unit.Positive;
    cancel: (time?: tone.Unit.TransportTime) => void;
    dispose: () => void;
    toSeconds: (time: tone.Unit.Time) => tone.Unit.Seconds;
    toFrequency: (frequency: tone.Unit.Frequency) => tone.Unit.Hertz;
}
declare abstract class BaseEventModel<T extends Event> extends NodeWithContextModel implements ObservableModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    protected abstract createEvent(): T;
    protected toSeconds(time: tone.Unit.Time): tone.Unit.Seconds;
    protected toFrequency(frequency: tone.Unit.Frequency): tone.Unit.Hertz;
    private getCallbackItems;
    private getToneCallback;
    private setCallback;
    getValueAtTime(traitName: string, _time: tone.Unit.Seconds): tone.BasicPlaybackState | tone.Unit.NormalRange;
    getValue(traitName: string): tone.BasicPlaybackState | tone.Unit.NormalRange;
    private play;
    protected handleMsg(command: any, _buffers: any): void;
    initEventListeners(): void;
    dispose(): void;
    event: T;
}
export declare class EventModel extends BaseEventModel<tone.ToneEvent> {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    protected createEvent(): tone.ToneEvent;
    initEventListeners(): void;
    dispose(): void;
    static model_name: string;
}
export declare class PartModel extends BaseEventModel<tone.Part> {
    defaults(): any;
    protected createEvent(): tone.Part;
    protected handleMsg(command: any, buffers: any): void;
    initEventListeners(): void;
    static model_name: string;
}
export declare class SequenceModel extends BaseEventModel<tone.Sequence> {
    defaults(): any;
    protected createEvent(): tone.Sequence;
    protected handleMsg(command: any, buffers: any): void;
    initEventListeners(): void;
    static model_name: string;
}
interface Loop extends Event {
    interval: tone.Unit.Time;
    iterations: number | null;
}
declare abstract class BaseLoopModel<T extends Loop> extends BaseEventModel<T> {
    defaults(): any;
    get iterations(): number;
    initEventListeners(): void;
}
export declare class LoopModel extends BaseLoopModel<tone.Loop> {
    defaults(): any;
    protected createEvent(): tone.Loop;
    static model_name: string;
}
export declare class PatternModel extends BaseLoopModel<tone.Pattern<any>> {
    defaults(): any;
    protected createEvent(): tone.Pattern<any>;
    initEventListeners(): void;
    static model_name: string;
}
export {};
