"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PatternModel = exports.LoopModel = exports.SequenceModel = exports.PartModel = exports.EventModel = void 0;
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
class BaseEventModel extends widget_base_1.NodeWithContextModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { value: null, humanize: false, probability: 1, mute: false, start_offset: 0, playback_rate: 1, loop_start: 0, loop_end: '1m', loop: false });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.event = this.createEvent();
    }
    toSeconds(time) {
        return this.event.toSeconds(time);
    }
    toFrequency(frequency) {
        return this.event.toFrequency(frequency);
    }
    // attach widget models to items
    async getCallbackItems(items) {
        const itemsModel = items.map(async (data) => {
            const model = await this.widget_manager.get_model(data.callee);
            const item = Object.assign({}, data);
            item.model = model;
            return item;
        });
        return Promise.all(itemsModel);
    }
    getToneCallback(items) {
        const callback = (time, value) => {
            items.forEach((item) => {
                const args = {};
                const skipArgs = [];
                let method = item.method;
                for (const [k, v] of Object.entries(item.args)) {
                    if (v.eval) {
                        args[k] = { value: eval(v.value), eval: true };
                    }
                    else {
                        args[k] = { value: v.value, eval: false };
                    }
                }
                // special case for the triggerNote method that doesn't exist in Tone.js:
                // method dispatch depending on the trigger_type value
                if (item.method === 'triggerNote') {
                    const trigger_type = args.trigger_type.value;
                    if (trigger_type === 'attack_release') {
                        method = 'triggerAttackRelease';
                    }
                    else {
                        if (trigger_type === 'attack') {
                            method = 'triggerAttack';
                        }
                        else {
                            method = 'triggerRelease';
                            skipArgs.push('velocity');
                            if (args.monophonic.value) {
                                skipArgs.push('note');
                            }
                        }
                        skipArgs.push('duration');
                    }
                    skipArgs.push('trigger_type');
                    skipArgs.push('monophonic');
                }
                const argsArray = (0, utils_1.normalizeArguments)(args, item.arg_keys, skipArgs);
                item.model.node[method](...argsArray);
            });
        };
        return callback;
    }
    async setCallback(command) {
        const items = await this.getCallbackItems(command.items);
        this.event.callback = this.getToneCallback(items);
    }
    getValueAtTime(traitName, _time) {
        return this.getValue(traitName);
    }
    getValue(traitName) {
        if (traitName === 'state') {
            return this.event.state;
        }
        else if (traitName === 'progress') {
            return this.event.progress;
        }
        else {
            throw new Error('unsupported trait name ' + traitName);
        }
    }
    play(command) {
        const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
        this.event[command.method](...argsArray);
    }
    handleMsg(command, _buffers) {
        if (command.event === 'set_callback') {
            this.setCallback(command);
        }
        else if (command.event === 'play') {
            this.play(command);
        }
        else if (command.event === 'cancel') {
            this.event.cancel(command.time);
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:humanize', () => {
            this.event.humanize = this.get('humanize');
        });
        this.on('change:probability', () => {
            this.event.probability = this.get('probability');
        });
        this.on('change:mute', () => {
            this.event.mute = this.get('mute');
        });
        this.on('change:playback_rate', () => {
            this.event.playbackRate = this.get('playback_rate');
        });
        this.on('msg:custom', this.handleMsg, this);
    }
    dispose() {
        this.event.dispose();
    }
}
class EventModel extends BaseEventModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: EventModel.model_name, value: null, loop_start: 0, loop_end: '1m', loop: false });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.event = this.createEvent();
    }
    createEvent() {
        return new tone.ToneEvent({
            value: this.get('value'),
            humanize: this.get('humanize'),
            probability: this.get('probability'),
            mute: this.get('mute'),
            playbackRate: this.get('playback_rate'),
            loopStart: this.get('loop_start'),
            loopEnd: this.get('loop_end'),
            loop: this.get('loop'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:value', () => {
            this.event.value = this.get('value');
        });
        this.on('change:loop_start', () => {
            this.event.loopStart = this.get('loop_start');
        });
        this.on('change:loop_end', () => {
            this.event.loopEnd = this.get('loop_end');
        });
        this.on('change:loop', () => {
            this.event.loop = this.get('loop');
        });
    }
    dispose() {
        this.event.dispose();
    }
}
exports.EventModel = EventModel;
EventModel.model_name = 'EventModel';
class PartModel extends BaseEventModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PartModel.model_name, _events: [], length: 0, loop_start: 0, loop_end: '1m', loop: false });
    }
    createEvent() {
        return new tone.Part({
            events: this.get('_events'),
            humanize: this.get('humanize'),
            probability: this.get('probability'),
            mute: this.get('mute'),
            playbackRate: this.get('playback_rate'),
            loopStart: this.get('loop_start'),
            loopEnd: this.get('loop_end'),
            loop: this.get('loop'),
        });
    }
    handleMsg(command, buffers) {
        super.handleMsg(command, buffers);
        if (command.event === 'add') {
            this.event.add(command.arg);
        }
        else if (command.event === 'at') {
            this.event.at(command.time, command.value);
        }
        else if (command.event === 'remove') {
            // TODO: not working in Tone.js?
            if (command.time === null) {
                console.log(command.value);
                console.log(this.event.at(command.value.time));
                this.event.remove(command.value);
            }
            else {
                console.log(command);
                console.log(this.event.at(command.time));
                this.event.remove(command.time, command.value);
            }
        }
        else if (command.event === 'clear') {
            this.event.clear();
        }
        // sync number of events
        this.set('length', this.event.length);
        this.save_changes();
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:loop_start', () => {
            this.event.loopStart = this.get('loop_start');
        });
        this.on('change:loop_end', () => {
            this.event.loopEnd = this.get('loop_end');
        });
        this.on('change:loop', () => {
            this.event.loop = this.get('loop');
        });
    }
}
exports.PartModel = PartModel;
PartModel.model_name = 'PartModel';
class SequenceModel extends BaseEventModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SequenceModel.model_name, events: [], _subdivision: '8n', length: 0, loop_start: 0, loop_end: 0, loop: true });
    }
    createEvent() {
        return new tone.Sequence({
            events: this.get('events'),
            subdivision: this.get('_subdivision'),
            humanize: this.get('humanize'),
            probability: this.get('probability'),
            mute: this.get('mute'),
            playbackRate: this.get('playback_rate'),
            loopStart: this.get('loop_start'),
            loopEnd: this.get('loop_end'),
            loop: this.get('loop'),
        });
    }
    handleMsg(command, buffers) {
        super.handleMsg(command, buffers);
        if (command.event === 'clear') {
            this.event.clear();
        }
        // sync number of events
        this.set('length', this.event.length);
        this.save_changes();
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:loop_start', () => {
            this.event.loopStart = this.get('loop_start');
        });
        this.on('change:loop_end', () => {
            this.event.loopEnd = this.get('loop_end');
        });
        this.on('change:loop', () => {
            this.event.loop = this.get('loop');
        });
        this.on('change:events', () => {
            this.event.events = this.get('events');
        });
    }
}
exports.SequenceModel = SequenceModel;
SequenceModel.model_name = 'SequenceModel';
class BaseLoopModel extends BaseEventModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { interval: '8n', iterations: null });
    }
    get iterations() {
        if (this.get('iterations') === null) {
            return Infinity;
        }
        else {
            return this.get('iterations');
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:interval', () => {
            this.event.interval = this.get('interval');
        });
        this.on('change:iterations', () => {
            this.event.iterations = this.iterations;
        });
    }
}
class LoopModel extends BaseLoopModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: LoopModel.model_name });
    }
    createEvent() {
        return new tone.Loop({
            interval: this.get('interval'),
            iterations: this.iterations,
            humanize: this.get('humanize'),
            probability: this.get('probability'),
            mute: this.get('mute'),
            playbackRate: this.get('playback_rate'),
        });
    }
}
exports.LoopModel = LoopModel;
LoopModel.model_name = 'LoopModel';
class PatternModel extends BaseLoopModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PatternModel.model_name, pattern: 'up', values: [] });
    }
    createEvent() {
        return new tone.Pattern({
            pattern: this.get('pattern'),
            values: this.get('values'),
            interval: this.get('interval'),
            iterations: this.iterations,
            humanize: this.get('humanize'),
            probability: this.get('probability'),
            mute: this.get('mute'),
            playbackRate: this.get('playback_rate'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:pattern', () => {
            this.event.pattern = this.get('pattern');
        });
        this.on('change:values', () => {
            this.event.values = this.get('values');
        });
    }
}
exports.PatternModel = PatternModel;
PatternModel.model_name = 'PatternModel';
//# sourceMappingURL=widget_event.js.map