"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AudioGraphModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const widget_base_1 = require("./widget_base");
function isNative(obj) {
    return obj instanceof AudioNode || obj instanceof AudioParam;
}
function isDisposed(obj) {
    return !isNative(obj) && obj.disposed;
}
function getConnectionNodes(connections) {
    return connections.map((conn) => {
        return [conn[0].node, conn[1].node, conn[2], conn[3]];
    });
}
function getConnectionId(conn) {
    return (conn[0].model_id + '::' + conn[1].model_id + '::' + conn[2] + '::' + conn[3]);
}
function getConnectionIds(connections) {
    return connections.map((conn) => {
        return getConnectionId(conn);
    });
}
class AudioGraphModel extends widget_base_1.ToneWidgetModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AudioGraphModel.model_name, _connections: [] });
    }
    get connections() {
        return this.get('_connections');
    }
    get connections_prev() {
        return this.previous('_connections');
    }
    updateConnections() {
        // connect nodes for new connections
        const connAdded = this.connections.filter((other) => {
            const other_id = getConnectionId(other);
            const prev_ids = getConnectionIds(this.connections_prev);
            return !prev_ids.includes(other_id);
        });
        getConnectionNodes(connAdded).forEach((conn_node) => {
            conn_node[0].connect(conn_node[1], conn_node[2], conn_node[3]);
        });
        // disconnect nodes for removed connections
        const connRemoved = this.connections_prev.filter((other) => {
            const other_id = getConnectionId(other);
            const current_ids = getConnectionIds(this.connections);
            return !current_ids.includes(other_id);
        });
        getConnectionNodes(connRemoved).forEach((conn_node) => {
            const src = conn_node[0];
            const dest = conn_node[1];
            // Disposed nodes should have been already disconnected by Tone.js
            if (!isDisposed(src) || !isDisposed(dest)) {
                const outputNum = conn_node[2];
                const inputNum = conn_node[3];
                src.disconnect(dest, outputNum, inputNum);
            }
        });
        // callbacks of updated destination nodes
        connAdded.concat(connRemoved).forEach((conn) => {
            conn[1].connectInputCallback();
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:_connections', this.updateConnections, this);
    }
}
exports.AudioGraphModel = AudioGraphModel;
AudioGraphModel.serializers = Object.assign(Object.assign({}, widget_base_1.ToneWidgetModel.serializers), { _connections: { deserialize: base_1.unpack_models } });
AudioGraphModel.model_name = 'AudioGraphModel';
//# sourceMappingURL=widget_graph.js.map