import { ISerializers } from '@jupyter-widgets/base';
import * as tone from 'tone';
import { AudioNodeModel } from './widget_base';
import { AudioBuffersModel, ParamModel } from './widget_core';
import { SignalModel } from './widget_signal';
interface InternalNodeModels {
    [name: string]: AudioNodeModel;
}
interface IpytoneInstrumentOptions {
    volume: tone.Unit.Decibels;
    triggerAttack: string;
    triggerRelease: string;
    afterInit: string;
    internalNodeModels: InternalNodeModels;
    createInternalNodes?: boolean;
    context?: tone.Context;
}
declare class IpytoneInstrument extends tone.PluckSynth {
    readonly name: string;
    private _triggerAttackFunc;
    private _triggerReleaseFunc;
    private _internalNodes;
    constructor(options: IpytoneInstrumentOptions);
    getNode(name: string): tone.ToneAudioNode | tone.Param;
    triggerAttack(note: tone.Unit.Frequency, time?: tone.Unit.Seconds, velocity?: tone.Unit.NormalRange): this;
    triggerRelease(...args: any[]): this;
    dispose(): this;
}
interface IpytoneMonophonicOptions extends IpytoneInstrumentOptions {
    setNote: string;
    getLevelAtTime: string;
    portamento: number;
    onsilence?: (instrument: tone.Synth) => void;
}
declare class IpytoneMonophonic extends tone.Synth {
    readonly name: string;
    readonly frequency: tone.Signal<'frequency'>;
    readonly detune: tone.Signal<'cents'>;
    private _triggerAttackFunc;
    private _triggerReleaseFunc;
    private _setNoteFunc;
    private _getLevelAtTimeFunc;
    private _internalNodes;
    constructor(options: IpytoneMonophonicOptions);
    getNode(name: string): tone.ToneAudioNode | tone.Param;
    _triggerEnvelopeAttack(time: tone.Unit.Seconds, velocity: tone.Unit.NormalRange): this;
    _triggerEnvelopeRelease(time: tone.Unit.Seconds): this;
    setNote(note: tone.Unit.Frequency, time?: tone.Unit.Time): this;
    getLevelAtTime(time: tone.Unit.Time): tone.Unit.NormalRange;
    set(props: any): this;
    dispose(): this;
}
declare abstract class BaseInstrumentModel<T extends IpytoneInstrument | IpytoneMonophonic> extends AudioNodeModel {
    defaults(): any;
    get volume(): ParamModel<'decibels'>;
    abstract createNode(): T;
    private handleMsg;
    initEventListeners(): void;
    node: T;
    static serializers: ISerializers;
}
export declare class InstrumentModel extends BaseInstrumentModel<IpytoneInstrument> {
    defaults(): any;
    createNode(): IpytoneInstrument;
    static model_name: string;
}
interface MonophonicSettings {
    [key: string]: string[];
}
export declare class MonophonicModel extends BaseInstrumentModel<IpytoneMonophonic> {
    defaults(): any;
    createNode(createInternalNodes?: boolean): IpytoneMonophonic;
    get nodeOptions(): IpytoneMonophonicOptions;
    get settings(): MonophonicSettings;
    get internalNodeModels(): InternalNodeModels;
    get detune(): SignalModel<'cents'>;
    initEventListeners(): void;
    static model_name: string;
}
export declare class PolySynthModel extends AudioNodeModel {
    defaults(): any;
    get dummyVoice(): MonophonicModel;
    createNode(): tone.PolySynth;
    private handleMsg;
    addSettingsEventListeners(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.PolySynth;
    static model_name: string;
}
export declare class SamplerModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.Sampler;
    get buffers(): AudioBuffersModel;
    private handleMsg;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.Sampler;
    static model_name: string;
}
export {};
