"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SamplerModel = exports.PolySynthModel = exports.MonophonicModel = exports.InstrumentModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
/*
 *  Get instrument's internal Tone.js nodes from their corresponding widget
 *  models.
 *
 *  If createNodes is true, it will create new Tone.js node instances from the
 *  corresponding models. This is used by PolySynthModel so that Tone.js PolySynth can
 *  create new instruments only in the front-end.
 */
function getInternalNodes(models, createNodes) {
    const nodes = {};
    Object.entries(models).forEach(([key, model]) => {
        if (createNodes) {
            if (model instanceof MonophonicModel) {
                nodes[key] = model.createNode(true);
            }
            else {
                nodes[key] = model.createNode();
            }
        }
        else {
            nodes[key] = model.node;
        }
    });
    return nodes;
}
// Hack: Tone.js abstract class ``Instrument`` is not exported
// So we have to inherit here from a subclass.
class IpytoneInstrument extends tone.PluckSynth {
    constructor(options) {
        super({ volume: options.volume });
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this.name = 'IpytoneInstrument';
        this._internalNodes = getInternalNodes(options.internalNodeModels, options.createInternalNodes);
        this._triggerAttackFunc = new Function('note', 'time', 'velocity', options.triggerAttack).bind(this);
        this._triggerReleaseFunc = new Function('time', options.triggerRelease).bind(this);
        const afterInit = new Function(options.afterInit).bind(this);
        afterInit();
        // Super hacky: dispose PluckSynth internal nodes that we won't use here.
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this._noise.dispose();
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this._lfcf.dispose();
    }
    /*
     * Allows getting internal (Tone) node instances by their name.
     */
    getNode(name) {
        return this._internalNodes[name];
    }
    triggerAttack(note, time, velocity) {
        this._triggerAttackFunc(note, time, velocity);
        return this;
    }
    triggerRelease(...args) {
        this._triggerReleaseFunc(...args);
        return this;
    }
    dispose() {
        super.dispose();
        Object.values(this._internalNodes).forEach((node) => {
            node.dispose();
        });
        return this;
    }
}
// Hack: Tone.js abstract class ``Monophonic`` is not exported
// So we have to inherit here from a subclass.
class IpytoneMonophonic extends tone.Synth {
    constructor(options) {
        super({
            volume: options.volume,
            portamento: options.portamento,
            onsilence: options.onsilence,
        });
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this.name = 'IpytoneMonophonic';
        if (this.onsilence === undefined) {
            this.onsilence = () => undefined;
        }
        this._internalNodes = getInternalNodes(options.internalNodeModels, options.createInternalNodes);
        this._triggerAttackFunc = new Function('time', 'velocity', options.triggerAttack).bind(this);
        this._triggerReleaseFunc = new Function('time', options.triggerRelease).bind(this);
        if (options.setNote !== '') {
            this._setNoteFunc = new Function('note', 'time', options.setNote).bind(this);
        }
        else {
            this._setNoteFunc = (note, time) => {
                super.setNote(note, time);
            };
        }
        this._getLevelAtTimeFunc = new Function('time', options.getLevelAtTime).bind(this);
        // Hack for re-setting readonly frequency and detune attributes
        // on a Tone.js Synth subclass
        Object.defineProperty(this, 'frequency', {
            writable: true,
        });
        Object.defineProperty(this, 'detune', {
            writable: true,
        });
        if ('frequency' in this._internalNodes) {
            this.frequency = this.getNode('frequency');
        }
        else {
            this.frequency = this.getNode('oscillator').frequency;
        }
        if ('detune' in this._internalNodes) {
            this.detune = this.getNode('detune');
        }
        else {
            this.detune = this.getNode('oscillator').detune;
        }
        Object.defineProperty(this, 'frequency', {
            writable: false,
        });
        Object.defineProperty(this, 'detune', {
            writable: false,
        });
        const afterInit = new Function(options.afterInit).bind(this);
        afterInit();
        // Super hacky: dispose Synth internal nodes that we won't use here.
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this.oscillator.dispose();
        // eslint-disable-next-line @typescript-eslint/ban-ts-comment
        //@ts-ignore
        this.envelope.dispose();
    }
    /*
     * Allows getting internal (Tone) node instances by their name.
     */
    getNode(name) {
        return this._internalNodes[name];
    }
    _triggerEnvelopeAttack(time, velocity) {
        this._triggerAttackFunc(time, velocity);
        return this;
    }
    _triggerEnvelopeRelease(time) {
        this._triggerReleaseFunc(time);
        return this;
    }
    setNote(note, time) {
        this._setNoteFunc(note, time);
        return this;
    }
    getLevelAtTime(time) {
        return this._getLevelAtTimeFunc(time);
    }
    /*
     * Overwrite the `set` function to allow updating settings of internal nodes.
     * This is used internally via PolySynth.
     */
    set(props) {
        Object.entries(props).forEach(([nodeName, nodeProps]) => {
            let node;
            if (nodeName === 'detune') {
                node = this.detune;
            }
            else {
                node = this.getNode(nodeName);
            }
            Object.entries(nodeProps).forEach(([propName, value]) => {
                // eslint-disable-next-line @typescript-eslint/ban-ts-comment
                //@ts-ignore
                //node.set({type: 'sine'});
                node.set({ [propName]: value });
            });
        });
        return this;
    }
    dispose() {
        super.dispose();
        Object.values(this._internalNodes).forEach((node) => {
            node.dispose();
        });
        return this;
    }
}
class BaseInstrumentModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _internal_nodes: {}, _after_init: '', _trigger_attack: '', _trigger_release: '' });
    }
    get volume() {
        return this.output.volume;
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('msg:custom', this.handleMsg, this);
    }
}
BaseInstrumentModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _internal_nodes: { deserialize: base_1.unpack_models } });
class InstrumentModel extends BaseInstrumentModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: InstrumentModel.model_name });
    }
    createNode() {
        return new IpytoneInstrument({
            volume: this.volume.value,
            triggerAttack: this.get('_trigger_attack'),
            triggerRelease: this.get('_trigger_release'),
            afterInit: this.get('_after_init'),
            internalNodeModels: this.get('_internal_nodes'),
        });
    }
}
exports.InstrumentModel = InstrumentModel;
InstrumentModel.model_name = 'InstrumentModel';
class MonophonicModel extends BaseInstrumentModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: MonophonicModel.model_name, _set_note: '', _get_level_at_time: '', _settings: {}, portamento: 0 });
    }
    /*
     * createInternalNodes = true allows to create a new IpytoneMonophonic
     * instance that itself encapsulates other IpytoneMonophonic instances
     * (e.g., a DuoSynth instance encapsulates two MonoSynth instrances).
     */
    createNode(createInternalNodes = false) {
        const options = this.nodeOptions;
        options.createInternalNodes = createInternalNodes;
        return new IpytoneMonophonic(options);
    }
    get nodeOptions() {
        return {
            volume: this.volume.value,
            triggerAttack: this.get('_trigger_attack'),
            triggerRelease: this.get('_trigger_release'),
            afterInit: this.get('_after_init'),
            setNote: this.get('_set_note'),
            getLevelAtTime: this.get('_get_level_at_time'),
            internalNodeModels: this.internalNodeModels,
            portamento: this.get('portamento'),
        };
    }
    get settings() {
        return this.get('_settings');
    }
    get internalNodeModels() {
        return this.get('_internal_nodes');
    }
    get detune() {
        if ('detune' in this.internalNodeModels) {
            return this.internalNodeModels['detune'];
        }
        else {
            return this.internalNodeModels['oscillator'].detune;
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:portamento', () => {
            this.node.portamento = this.get('portamento');
        });
    }
}
exports.MonophonicModel = MonophonicModel;
MonophonicModel.model_name = 'MonophonicModel';
class PolySynthModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PolySynthModel.model_name, _dummy_voice: null, max_polyphony: 32 });
    }
    get dummyVoice() {
        return this.get('_dummy_voice');
    }
    createNode() {
        const options = this.dummyVoice.nodeOptions;
        options.createInternalNodes = true;
        return new tone.PolySynth({
            maxPolyphony: this.get('max_polyphony'),
            voice: IpytoneMonophonic,
            options: options,
        });
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
    /*
     * Add event listeners that allow to update each voice of the
     * PolySynth when any setting of its dummy voice is updated.
     */
    addSettingsEventListeners() {
        const detuneModel = this.dummyVoice.detune;
        detuneModel.on('change:value', () => {
            const props = {};
            props['detune'] = {};
            props['detune']['value'] = detuneModel.get('value');
            this.node.set(props);
        });
        const settings = this.dummyVoice.settings;
        Object.entries(settings).forEach(([nodeName, propNames]) => {
            const nodeModel = this.dummyVoice.internalNodeModels[nodeName];
            Object.values(propNames).forEach((propName) => {
                nodeModel.on(`change:${propName}`, () => {
                    const props = {};
                    props[nodeName] = {};
                    props[nodeName][propName] = nodeModel.get(propName);
                    this.node.set(props);
                });
            });
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.addSettingsEventListeners();
        this.on('change:max_polyphony', () => {
            this.node.maxPolyphony = this.get('max_polyphony');
        });
        this.on('msg:custom', this.handleMsg, this);
    }
}
exports.PolySynthModel = PolySynthModel;
PolySynthModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _dummy_voice: { deserialize: base_1.unpack_models } });
PolySynthModel.model_name = 'PolySynthModel';
class SamplerModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SamplerModel.model_name, _buffers: null, attack: 0, release: 1, curve: 'exponential' });
    }
    createNode() {
        const sampler = new tone.Sampler({
            attack: this.get('attack'),
            release: this.get('release'),
            curve: this.get('curve'),
            urls: this.buffers.buffer_nodes,
        });
        // Hack: allows reusing AudioBuffers widget from the Python side
        this.buffers.setNode(sampler._buffers);
        return sampler;
    }
    get buffers() {
        return this.get('_buffers');
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:attack', () => {
            this.node.attack = this.get('attack');
        });
        this.on('change:release', () => {
            this.node.release = this.get('release');
        });
        this.on('change:curve', () => {
            this.node.curve = this.get('curve');
        });
        this.on('msg:custom', this.handleMsg, this);
    }
}
exports.SamplerModel = SamplerModel;
SamplerModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _buffers: { deserialize: base_1.unpack_models } });
SamplerModel.model_name = 'SamplerModel';
//# sourceMappingURL=widget_instrument.js.map