/// <reference types="backbone" />
import { ISerializers } from '@jupyter-widgets/base';
import * as tone from 'tone';
import { UnitName, UnitMap } from 'tone/Tone/core/type/Units';
import { AudioNodeModel } from './widget_base';
import { ParamModel } from './widget_core';
import { ObservableModel } from './widget_observe';
declare abstract class SignalOperatorModel extends AudioNodeModel {
    defaults(): any;
    static model_name: string;
}
export declare class SignalModel<T extends UnitName> extends SignalOperatorModel implements ObservableModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    get input(): ParamModel<T>;
    get value(): UnitMap[T];
    createNode(): tone.Signal<T>;
    private updateOverridden;
    connectInputCallback(): void;
    getValueAtTime(traitName: string, time: tone.Unit.Seconds): UnitMap[T];
    getValue(traitName: string): UnitMap[T];
    private handleMsg;
    initEventListeners(): void;
    node: tone.Signal<T>;
    static model_name: string;
}
export declare class MultiplyModel extends SignalModel<'number'> {
    defaults(): any;
    createNode(): tone.Multiply;
    get factor(): ParamModel<'number'>;
    static serializers: ISerializers;
    node: tone.Multiply;
    static model_name: string;
}
export declare class AddModel extends SignalModel<'number'> {
    defaults(): any;
    createNode(): tone.Add;
    setSubNodes(): void;
    get addend(): ParamModel<'number'>;
    static serializers: ISerializers;
    node: tone.Add;
    static model_name: string;
}
export declare class SubtractModel extends SignalModel<'number'> {
    defaults(): any;
    createNode(): tone.Subtract;
    get subtrahend(): ParamModel<'number'>;
    setSubNodes(): void;
    static serializers: ISerializers;
    node: tone.Subtract;
    static model_name: string;
}
export declare class GreaterThanModel extends SignalModel<'number'> {
    defaults(): any;
    createNode(): tone.GreaterThan;
    get comparator(): ParamModel<'number'>;
    setSubNodes(): void;
    static serializers: ISerializers;
    node: tone.GreaterThan;
    static model_name: string;
}
export declare class AudioToGainModel extends SignalOperatorModel {
    defaults(): any;
    createNode(): tone.AudioToGain;
    node: tone.AudioToGain;
    static model_name: string;
}
export declare class AbsModel extends SignalOperatorModel {
    defaults(): any;
    createNode(): tone.Abs;
    node: tone.Abs;
    static model_name: string;
}
export declare class NegateModel extends SignalOperatorModel {
    defaults(): any;
    createNode(): tone.Negate;
    node: tone.Negate;
    static model_name: string;
}
export declare class PowModel extends SignalOperatorModel {
    defaults(): any;
    createNode(): tone.Pow;
    initEventListeners(): void;
    node: tone.Pow;
    static model_name: string;
}
export declare class ScaleModel extends SignalOperatorModel {
    defaults(): any;
    createNode(): tone.Scale;
    initEventListeners(): void;
    node: tone.Scale;
    static model_name: string;
}
export declare class WaveShaperModel extends SignalOperatorModel {
    defaults(): any;
    get curve(): Float32Array | null;
    createNode(): tone.WaveShaper;
    initEventListeners(): void;
    node: tone.WaveShaper;
    static serializers: ISerializers;
    static model_name: string;
}
export {};
