"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WaveShaperModel = exports.ScaleModel = exports.PowModel = exports.NegateModel = exports.AbsModel = exports.AudioToGainModel = exports.GreaterThanModel = exports.SubtractModel = exports.AddModel = exports.MultiplyModel = exports.SignalModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const tone = __importStar(require("tone"));
const serializers_1 = require("./serializers");
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
class SignalOperatorModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SignalOperatorModel.model_name });
    }
}
SignalOperatorModel.model_name = 'SignalOperatorModel';
class SignalModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SignalModel.model_name, _override: true });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        if (this.get('_create_node')) {
            this.updateOverridden();
        }
    }
    get input() {
        return this.get('_input');
    }
    get value() {
        return this.get('value');
    }
    createNode() {
        return new tone.Signal(this.input.properties);
    }
    updateOverridden() {
        this.input.set('overridden', this.node.overridden);
        // if overridden, value is reset to 0
        this.input.set('value', this.node.value);
        this.input.save_changes();
        this.set('value', this.node.value);
        this.save_changes();
    }
    connectInputCallback() {
        // new connected incoming signal overrides this signal value
        this.updateOverridden();
    }
    getValueAtTime(traitName, time) {
        (0, utils_1.assert)(traitName === 'value', 'param only supports "value" trait');
        return this.node.getValueAtTime(time);
    }
    getValue(traitName) {
        (0, utils_1.assert)(traitName === 'value', 'param only supports "value" trait');
        return this.node.value;
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:value', () => {
            this.node.value = this.get('value');
        });
        this.on('msg:custom', this.handleMsg, this);
    }
}
exports.SignalModel = SignalModel;
SignalModel.model_name = 'SignalModel';
class MultiplyModel extends SignalModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: MultiplyModel.model_name, _factor: undefined });
    }
    createNode() {
        return new tone.Multiply(this.factor.value);
    }
    get factor() {
        return this.get('_factor');
    }
}
exports.MultiplyModel = MultiplyModel;
MultiplyModel.serializers = Object.assign(Object.assign({}, SignalModel.serializers), { _factor: { deserialize: base_1.unpack_models } });
MultiplyModel.model_name = 'MultiplyModel';
class AddModel extends SignalModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AddModel.model_name, _addend: undefined });
    }
    createNode() {
        return new tone.Add(this.addend.value);
    }
    setSubNodes() {
        super.setSubNodes();
        this.addend.setNode(this.node.addend);
    }
    get addend() {
        return this.get('_addend');
    }
}
exports.AddModel = AddModel;
AddModel.serializers = Object.assign(Object.assign({}, SignalModel.serializers), { _addend: { deserialize: base_1.unpack_models } });
AddModel.model_name = 'AddModel';
class SubtractModel extends SignalModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SubtractModel.model_name, _subtrahend: undefined });
    }
    createNode() {
        return new tone.Subtract(this.subtrahend.value);
    }
    get subtrahend() {
        return this.get('_subtrahend');
    }
    setSubNodes() {
        super.setSubNodes();
        this.subtrahend.setNode(this.node.subtrahend);
    }
}
exports.SubtractModel = SubtractModel;
SubtractModel.serializers = Object.assign(Object.assign({}, SignalModel.serializers), { _subtrahend: { deserialize: base_1.unpack_models } });
SubtractModel.model_name = 'SubtractModel';
class GreaterThanModel extends SignalModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: GreaterThanModel.model_name, _comparator: undefined });
    }
    createNode() {
        return new tone.GreaterThan(this.comparator.value);
    }
    get comparator() {
        return this.get('_comparator');
    }
    setSubNodes() {
        super.setSubNodes();
        this.comparator.setNode(this.node.comparator);
    }
}
exports.GreaterThanModel = GreaterThanModel;
GreaterThanModel.serializers = Object.assign(Object.assign({}, SignalModel.serializers), { _comparator: { deserialize: base_1.unpack_models } });
GreaterThanModel.model_name = 'GreaterThanModel';
class AudioToGainModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AudioToGainModel.model_name });
    }
    createNode() {
        return new tone.AudioToGain();
    }
}
exports.AudioToGainModel = AudioToGainModel;
AudioToGainModel.model_name = 'AudioToGainModel';
class AbsModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AbsModel.model_name });
    }
    createNode() {
        return new tone.Abs();
    }
}
exports.AbsModel = AbsModel;
AbsModel.model_name = 'AbsModel';
class NegateModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: NegateModel.model_name });
    }
    createNode() {
        return new tone.Negate();
    }
}
exports.NegateModel = NegateModel;
NegateModel.model_name = 'NegateModel';
class PowModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PowModel.model_name, value: 1 });
    }
    createNode() {
        return new tone.Pow(this.get('value'));
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:value', () => {
            this.node.value = this.get('value');
        });
    }
}
exports.PowModel = PowModel;
PowModel.model_name = 'PowModel';
class ScaleModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: ScaleModel.model_name, min_out: 0.0, max_out: 1.0 });
    }
    createNode() {
        return new tone.Scale(this.get('min_out'), this.get('max_out'));
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:min_out', () => {
            this.node.min = this.get('min_out');
        });
        this.on('change:max_out', () => {
            this.node.min = this.get('max_out');
        });
    }
}
exports.ScaleModel = ScaleModel;
ScaleModel.model_name = 'ScaleModel';
class WaveShaperModel extends SignalOperatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: WaveShaperModel.model_name, curve: null, oversample: 'none' });
    }
    get curve() {
        // array should be 1-dimensional
        return (0, serializers_1.getArrayProp)(this.get('curve'));
    }
    createNode() {
        let data;
        data = this.curve;
        if (data === null) {
            data = [-1, 0, 1];
        }
        const node = new tone.WaveShaper(data);
        node.oversample = this.get('oversample');
        return node;
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:curve', () => {
            this.node.curve = this.curve;
        });
        this.on('change:oversample', () => {
            this.node.oversample = this.get('oversample');
        });
    }
}
exports.WaveShaperModel = WaveShaperModel;
WaveShaperModel.serializers = Object.assign(Object.assign({}, SignalOperatorModel.serializers), { curve: serializers_1.dataarray_serialization });
WaveShaperModel.model_name = 'WaveShaperModel';
//# sourceMappingURL=widget_signal.js.map