/// <reference types="backbone" />
import { ISerializers } from '@jupyter-widgets/base';
import * as tone from 'tone';
import { AudioNodeModel } from './widget_base';
import { AudioBufferModel, AudioBuffersModel, ParamModel } from './widget_core';
import { ObservableModel } from './widget_observe';
import { SignalModel, MultiplyModel } from './widget_signal';
import { ArrayProperty } from './serializers';
interface SourceInterface extends tone.ToneAudioNode {
    mute: boolean;
    volume: tone.Param<'decibels'>;
    state: tone.BasicPlaybackState;
    sync(): this;
    unsync(): this;
}
declare abstract class SourceModel extends AudioNodeModel implements ObservableModel {
    defaults(): any;
    get mute(): boolean;
    get volume(): ParamModel<'decibels'>;
    getValueAtTime(traitName: string, _time: tone.Unit.Seconds): tone.BasicPlaybackState;
    getValue(traitName: string): tone.BasicPlaybackState;
    initEventListeners(): void;
    private handleMsg;
    static serializers: ISerializers;
    node: SourceInterface;
    static model_name: string;
}
export declare class LFOModel extends AudioNodeModel {
    defaults(): any;
    get frequency(): SignalModel<'frequency'>;
    get amplitude(): ParamModel<'normalRange'>;
    createNode(): tone.LFO;
    setSubNodes(): void;
    initEventListeners(): void;
    private handleMsg;
    static serializers: ISerializers;
    node: tone.LFO;
    static model_name: string;
}
interface BaseOscillatorInterface extends SourceInterface {
    frequency: tone.Signal<'frequency'>;
    detune: tone.Signal<'cents'>;
    type: string;
    partials: number[];
    phase: number;
    asArray(length: number): Promise<Float32Array>;
}
declare abstract class BaseOscillatorModel extends SourceModel {
    defaults(): any;
    initialize(attributes: Backbone.ObjectHash, options: {
        model_id: string;
        comm: any;
        widget_manager: any;
    }): void;
    get frequency(): SignalModel<'frequency'>;
    get detune(): SignalModel<'cents'>;
    get array(): ArrayProperty;
    set array(value: ArrayProperty);
    setSubNodes(): void;
    maybeSetArray(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: BaseOscillatorInterface;
    static model_name: string;
}
export declare class OscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    createNode(): tone.Oscillator;
    node: tone.Oscillator;
    static model_name: string;
}
export declare class AMOscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    get harmonicity(): MultiplyModel;
    createNode(): tone.AMOscillator;
    setSubNodes(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.AMOscillator;
    static model_name: string;
}
export declare class FMOscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    get harmonicity(): MultiplyModel;
    get modulationIndex(): MultiplyModel;
    createNode(): tone.FMOscillator;
    setSubNodes(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.FMOscillator;
    static model_name: string;
}
export declare class FatOscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    createNode(): tone.FatOscillator;
    initEventListeners(): void;
    node: tone.FatOscillator;
    static model_name: string;
}
export declare class PulseOscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    get width(): SignalModel<'audioRange'>;
    createNode(): tone.PulseOscillator;
    setSubNodes(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.PulseOscillator;
    static model_name: string;
}
export declare class PWMOscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    get modulation_frequency(): SignalModel<'frequency'>;
    createNode(): tone.PWMOscillator;
    setSubNodes(): void;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.PWMOscillator;
    static model_name: string;
}
type AnyOscillator = tone.Oscillator | tone.PWMOscillator | tone.PulseOscillator | tone.FatOscillator | tone.AMOscillator | tone.FMOscillator;
export declare class OmniOscillatorModel extends BaseOscillatorModel {
    defaults(): any;
    get modulation_frequency(): SignalModel<'frequency'>;
    get width(): SignalModel<'audioRange'>;
    get harmonicity(): MultiplyModel;
    get modulationIndex(): MultiplyModel;
    createNode(): tone.OmniOscillator<AnyOscillator>;
    setSubNodes(): void;
    private setOptionalNodes;
    initEventListeners(): void;
    static serializers: ISerializers;
    node: tone.OmniOscillator<AnyOscillator>;
    static model_name: string;
}
export declare class NoiseModel extends SourceModel {
    defaults(): any;
    createNode(): tone.Noise;
    get type(): tone.NoiseType;
    get fadeIn(): number;
    get fadeOut(): number;
    initEventListeners(): void;
    node: tone.Noise;
    static model_name: string;
}
export declare class PlayerModel extends SourceModel {
    defaults(): any;
    createNode(): tone.Player;
    get buffer(): AudioBufferModel;
    initEventListeners(): void;
    node: tone.Player;
    static serializers: ISerializers;
    static model_name: string;
}
export declare class PlayersModel extends AudioNodeModel {
    defaults(): any;
    createNode(): tone.Players;
    get mute(): boolean;
    get volume(): ParamModel<'decibels'>;
    get buffers(): AudioBuffersModel;
    initEventListeners(): void;
    node: tone.Players;
    static serializers: ISerializers;
    static model_name: string;
}
export {};
