"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlayersModel = exports.PlayerModel = exports.NoiseModel = exports.OmniOscillatorModel = exports.PWMOscillatorModel = exports.PulseOscillatorModel = exports.FatOscillatorModel = exports.FMOscillatorModel = exports.AMOscillatorModel = exports.OscillatorModel = exports.LFOModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
const serializers_1 = require("./serializers");
class SourceModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: SourceModel.model_name, _volume: undefined, mute: false });
    }
    get mute() {
        return this.get('mute');
    }
    get volume() {
        return this.get('_volume');
    }
    getValueAtTime(traitName, _time) {
        return this.getValue(traitName);
    }
    getValue(traitName) {
        (0, utils_1.assert)(traitName === 'state', 'param only supports "state" trait');
        return this.node.state;
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:mute', () => {
            this.node.mute = this.mute;
            // update volume param model value
            this.volume.value = this.node.volume.value;
            this.volume.save_changes();
        });
        this.on('msg:custom', this.handleMsg, this);
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
}
SourceModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _volume: { deserialize: base_1.unpack_models } });
SourceModel.model_name = 'SourceModel';
class LFOModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: LFOModel.model_name, _frequency: undefined, _amplitude: undefined, type: 'sine', partials: [], phase: 0, units: 'number', convert: true });
    }
    get frequency() {
        return this.get('_frequency');
    }
    get amplitude() {
        return this.get('_amplitude');
    }
    createNode() {
        return new tone.LFO({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            amplitude: this.get('_amplitude').get('value'),
            min: this.get('_output').get('min_out'),
            max: this.get('_output').get('max_out'),
            phase: this.get('phase'),
            partials: this.get('_partials'),
            units: this.get('units'),
        });
    }
    setSubNodes() {
        this.frequency.setNode(this.node.frequency);
        this.amplitude.setNode(this.node.amplitude);
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:type', () => {
            this.node.type = this.get('type');
        });
        this.on('change:partials', () => {
            this.node.partials = this.get('partials');
        });
        this.on('change:phase', () => {
            this.node.phase = this.get('phase');
        });
        this.on('change:units', () => {
            this.node.units = this.get('units');
        });
        this.on('change:convert', () => {
            this.node.convert = this.get('convert');
        });
        this.on('msg:custom', this.handleMsg, this);
    }
    handleMsg(command, _buffers) {
        if (command.event === 'trigger') {
            const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
            this.node[command.method](...argsArray);
        }
    }
}
exports.LFOModel = LFOModel;
LFOModel.serializers = Object.assign(Object.assign({}, widget_base_1.AudioNodeModel.serializers), { _frequency: { deserialize: base_1.unpack_models }, _amplitude: { deserialize: base_1.unpack_models } });
LFOModel.model_name = 'LFOModel';
class BaseOscillatorModel extends SourceModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: BaseOscillatorModel.model_name, type: 'sine', _partials: [], _frequency: null, _detune: null, phase: 0, array: null, array_length: 1024, sync_array: false });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        if (this.get('_create_node')) {
            this.maybeSetArray();
        }
    }
    get frequency() {
        return this.get('_frequency');
    }
    get detune() {
        return this.get('_detune');
    }
    get array() {
        return (0, serializers_1.getArrayProp)(this.get('array'));
    }
    set array(value) {
        // avoid infinite event listener loop
        this.set('array', value, { silent: true });
    }
    setSubNodes() {
        this.frequency.setNode(this.node.frequency);
        this.detune.setNode(this.node.detune);
    }
    maybeSetArray() {
        if (this.get('sync_array')) {
            this.node.asArray(this.get('array_length')).then((arr) => {
                this.array = arr;
                this.save_changes();
            });
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:type', () => {
            // partials changed -> tone updates the type)
            if (!this.get('type').endsWith('custom')) {
                this.node.type = this.get('type');
                this.maybeSetArray();
            }
        });
        this.on('change:_partials', () => {
            this.node.partials = this.get('_partials');
            this.maybeSetArray();
        });
        this.on('change:phase', () => {
            this.node.phase = this.get('phase');
            this.maybeSetArray();
        });
    }
}
BaseOscillatorModel.serializers = Object.assign(Object.assign({}, SourceModel.serializers), { _frequency: { deserialize: base_1.unpack_models }, _detune: { deserialize: base_1.unpack_models }, array: serializers_1.dataarray_serialization });
BaseOscillatorModel.model_name = 'BaseOscillatorModel';
class OscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: OscillatorModel.model_name });
    }
    createNode() {
        return new tone.Oscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
            partials: this.get('_partials'),
        });
    }
}
exports.OscillatorModel = OscillatorModel;
OscillatorModel.model_name = 'OscillatorModel';
class AMOscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: AMOscillatorModel.model_name, _harmonicity: null, modulation_type: 'square' });
    }
    get harmonicity() {
        return this.get('_harmonicity');
    }
    createNode() {
        return new tone.AMOscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            harmonicity: this.get('_harmonicity').get('value'),
            modulationType: this.get('modulation_type'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.harmonicity.setNode(this.node.harmonicity);
    }
    initEventListeners() {
        super.initEventListeners();
        this.harmonicity.on('change:value', () => {
            this.maybeSetArray();
        });
        this.on('change:modulation_type', () => {
            this.node.modulationType = this.get('modulation_type');
            this.maybeSetArray();
        });
    }
}
exports.AMOscillatorModel = AMOscillatorModel;
AMOscillatorModel.serializers = Object.assign(Object.assign({}, BaseOscillatorModel.serializers), { _harmonicity: { deserialize: base_1.unpack_models } });
AMOscillatorModel.model_name = 'AMOscillatorModel';
class FMOscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: FMOscillatorModel.model_name, _harmonicity: null, _modulation_index: null, modulation_type: 'square' });
    }
    get harmonicity() {
        return this.get('_harmonicity');
    }
    get modulationIndex() {
        return this.get('_modulation_index');
    }
    createNode() {
        return new tone.FMOscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            harmonicity: this.get('_harmonicity').get('value'),
            modulationType: this.get('modulation_type'),
            modulationIndex: this.get('_modulation_index').get('value'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.harmonicity.setNode(this.node.harmonicity);
        this.modulationIndex.setNode(this.node.modulationIndex);
    }
    initEventListeners() {
        super.initEventListeners();
        this.harmonicity.on('change:value', () => {
            this.maybeSetArray();
        });
        this.modulationIndex.on('change:value', () => {
            this.maybeSetArray();
        });
        this.on('change:modulation_type', () => {
            this.node.modulationType = this.get('modulation_type');
            this.maybeSetArray();
        });
    }
}
exports.FMOscillatorModel = FMOscillatorModel;
FMOscillatorModel.serializers = Object.assign(Object.assign({}, BaseOscillatorModel.serializers), { _harmonicity: { deserialize: base_1.unpack_models }, _modulation_index: { deserialize: base_1.unpack_models } });
FMOscillatorModel.model_name = 'FMOscillatorModel';
class FatOscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: FatOscillatorModel.model_name, spread: 20, count: 3 });
    }
    createNode() {
        return new tone.FatOscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
            partials: this.get('_partials'),
            spread: this.get('spread'),
            count: this.get('count'),
        });
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:spread', () => {
            this.node.spread = this.get('spread');
            this.maybeSetArray();
        });
        this.on('change:count', () => {
            this.node.count = this.get('count');
            this.maybeSetArray();
        });
    }
}
exports.FatOscillatorModel = FatOscillatorModel;
FatOscillatorModel.model_name = 'FatOscillatorModel';
class PulseOscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PulseOscillatorModel.model_name, _width: null });
    }
    get width() {
        return this.get('_width');
    }
    createNode() {
        return new tone.PulseOscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            width: this.get('_width').get('value'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.width.setNode(this.node.width);
    }
    initEventListeners() {
        super.initEventListeners();
        this.width.on('change:value', () => {
            this.maybeSetArray();
        });
    }
}
exports.PulseOscillatorModel = PulseOscillatorModel;
PulseOscillatorModel.serializers = Object.assign(Object.assign({}, BaseOscillatorModel.serializers), { _width: { deserialize: base_1.unpack_models } });
PulseOscillatorModel.model_name = 'PulseOscillatorModel';
class PWMOscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PWMOscillatorModel.model_name, _modulation_frequency: null });
    }
    get modulation_frequency() {
        return this.get('_modulation_frequency');
    }
    createNode() {
        return new tone.PWMOscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            modulationFrequency: this.get('_modulation_frequency').get('value'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.modulation_frequency.setNode(this.node.modulationFrequency);
    }
    initEventListeners() {
        super.initEventListeners();
        this.modulation_frequency.on('change:value', () => {
            this.maybeSetArray();
        });
    }
}
exports.PWMOscillatorModel = PWMOscillatorModel;
PWMOscillatorModel.serializers = Object.assign(Object.assign({}, BaseOscillatorModel.serializers), { _modulation_frequency: { deserialize: base_1.unpack_models } });
PWMOscillatorModel.model_name = 'PWMOscillatorModel';
class OmniOscillatorModel extends BaseOscillatorModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: OmniOscillatorModel.model_name, _modulation_frequency: null, _width: null, _harmonicity: null, _modulation_index: null, modulation_type: 'square', spread: 20, count: 3 });
    }
    get modulation_frequency() {
        return this.get('_modulation_frequency');
    }
    get width() {
        return this.get('_width');
    }
    get harmonicity() {
        return this.get('_harmonicity');
    }
    get modulationIndex() {
        return this.get('_modulation_index');
    }
    createNode() {
        return new tone.OmniOscillator({
            type: this.get('type'),
            frequency: this.get('_frequency').get('value'),
            detune: this.get('_detune').get('value'),
            volume: this.get('_volume').get('value'),
            phase: this.get('phase'),
        });
    }
    setSubNodes() {
        super.setSubNodes();
        this.setOptionalNodes();
    }
    setOptionalNodes() {
        if (this.node.sourceType === 'am' || this.node.sourceType === 'fm') {
            this.harmonicity.setNode(this.node.harmonicity);
            this.node.harmonicity.value =
                this.get('_harmonicity').get('value');
            if (this.node.sourceType === 'fm') {
                this.modulationIndex.setNode(this.node.modulationIndex);
                this.node.modulationIndex.value =
                    this.get('_modulation_index').get('value');
                this.node.modulationType = this.get('_modulation_type');
            }
        }
        else if (this.node.sourceType === 'fat') {
            this.node.spread = this.get('spread');
            this.node.count = this.get('count');
        }
        else if (this.node.sourceType === 'pulse') {
            this.width.setNode(this.node.width);
            this.node.width.value =
                this.get('_width').get('value');
        }
        else if (this.node.sourceType === 'pwm') {
            this.modulation_frequency.setNode(this.node.modulationFrequency);
            this.node.modulationFrequency.value =
                this.get('_modulation_frequency').get('value');
        }
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:type', () => {
            this.setOptionalNodes();
        });
        this.modulation_frequency.on('change:value', () => {
            this.maybeSetArray();
        });
        this.width.on('change:value', () => {
            this.maybeSetArray();
        });
        this.harmonicity.on('change:value', () => {
            this.maybeSetArray();
        });
        this.modulationIndex.on('change:value', () => {
            this.maybeSetArray();
        });
        this.on('change:modulation_type', () => {
            this.node.modulationType = this.get('modulation_type');
            this.maybeSetArray();
        });
        this.on('change:spread', () => {
            this.node.spread = this.get('spread');
            this.maybeSetArray();
        });
        this.on('change:count', () => {
            this.node.count = this.get('count');
            this.maybeSetArray();
        });
    }
}
exports.OmniOscillatorModel = OmniOscillatorModel;
OmniOscillatorModel.serializers = Object.assign(Object.assign({}, BaseOscillatorModel.serializers), { _modulation_frequency: { deserialize: base_1.unpack_models }, _width: { deserialize: base_1.unpack_models }, _harmonicity: { deserialize: base_1.unpack_models }, _modulation_index: { deserialize: base_1.unpack_models } });
OmniOscillatorModel.model_name = 'OmniOscillatorModel';
class NoiseModel extends SourceModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: NoiseModel.model_name, type: 'white', fade_in: 0, fade_out: 0 });
    }
    createNode() {
        return new tone.Noise({
            type: this.get('type'),
            volume: this.get('_volume').get('value'),
            fadeIn: this.get('fade_in'),
            fadeOut: this.get('fade_out'),
        });
    }
    get type() {
        return this.get('type');
    }
    get fadeIn() {
        return this.get('fade_in');
    }
    get fadeOut() {
        return this.get('fade_out');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:type', () => {
            this.node.type = this.type;
        });
        this.on('change:fade_in', () => {
            this.node.fadeIn = this.fadeIn;
        });
        this.on('change:fade_out', () => {
            this.node.fadeOut = this.fadeOut;
        });
    }
}
exports.NoiseModel = NoiseModel;
NoiseModel.model_name = 'NoiseModel';
class PlayerModel extends SourceModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PlayerModel.model_name, buffer: null, autostart: false, loop: false, loop_start: 0, loop_end: 0, fade_in: 0, fade_out: 0, reverse: false, playback_rate: 1 });
    }
    createNode() {
        return new tone.Player({
            url: this.buffer.node,
            autostart: this.get('autostart'),
            loop: this.get('loop'),
            loopEnd: this.get('loop_end'),
            loopStart: this.get('loop_start'),
            fadeIn: this.get('fade_in'),
            fadeOut: this.get('fade_out'),
            reverse: this.get('reverse'),
            playbackRate: this.get('playback_rate'),
            volume: this.get('_volume').get('value'),
        });
    }
    get buffer() {
        return this.get('buffer');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:buffer', () => {
            this.node.buffer = this.buffer.node;
        });
        this.on('change:loop', () => {
            this.node.loop = this.get('loop');
        });
        this.on('change:loop_end', () => {
            this.node.loopEnd = this.get('loop_end');
        });
        this.on('change:loop_start', () => {
            this.node.loopStart = this.get('loop_start');
        });
        this.on('change:fade_in', () => {
            this.node.fadeIn = this.get('fade_in');
        });
        this.on('change:fade_out', () => {
            this.node.fadeOut = this.get('fade_out');
        });
        this.on('change:playback_rate', () => {
            this.node.playbackRate = this.get('playback_rate');
        });
        this.on('change:reverse', () => {
            this.node.reverse = this.get('reverse');
            this.buffer.set('reverse', this.buffer.node.reverse, { silent: true });
            this.buffer.save_changes();
        });
    }
}
exports.PlayerModel = PlayerModel;
PlayerModel.serializers = Object.assign(Object.assign({}, SourceModel.serializers), { buffer: { deserialize: base_1.unpack_models } });
PlayerModel.model_name = 'PlayerModel';
class PlayersModel extends widget_base_1.AudioNodeModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: PlayersModel.model_name, _buffers: null, _volume: undefined, mute: false });
    }
    createNode() {
        const players = new tone.Players({
            urls: this.buffers.buffer_nodes,
            fadeIn: this.get('fade_in'),
            fadeOut: this.get('fade_out'),
            volume: this.get('_volume').get('value'),
            mute: this.mute,
        });
        // Hack: allows reusing AudioBuffers widget from the Python side
        this.buffers.setNode(players._buffers);
        return players;
    }
    get mute() {
        return this.get('mute');
    }
    get volume() {
        return this.get('_volume');
    }
    get buffers() {
        return this.get('_buffers');
    }
    initEventListeners() {
        super.initEventListeners();
        this.on('change:mute', () => {
            this.node.mute = this.mute;
            // update volume param model value
            this.volume.value = this.node.volume.value;
            this.volume.save_changes();
        });
    }
}
exports.PlayersModel = PlayersModel;
PlayersModel.serializers = Object.assign(Object.assign({}, SourceModel.serializers), { _buffers: { deserialize: base_1.unpack_models } });
PlayersModel.model_name = 'PlayersModel';
//# sourceMappingURL=widget_source.js.map