"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransportModel = void 0;
const base_1 = require("@jupyter-widgets/base");
const tone = __importStar(require("tone"));
const utils_1 = require("./utils");
const widget_base_1 = require("./widget_base");
class TransportModel extends widget_base_1.ToneObjectModel {
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { _model_name: TransportModel.model_name, _bpm: null, time_signature: 4, state: 'stopped', loop_start: 0, loop_end: '4m', loop: false, swing: 0, swing_subdivision: '8n', position: 0, seconds: 0, progress: 0, ticks: 0 });
    }
    initialize(attributes, options) {
        super.initialize(attributes, options);
        this.py2jsEventID = {};
        this.bpm.setNode(tone.Transport.bpm);
    }
    get bpm() {
        return this.get('_bpm');
    }
    toSeconds(time) {
        return tone.Transport.toSeconds(time);
    }
    toFrequency(frequency) {
        return tone.Transport.toFrequency(frequency);
    }
    // attach widget models to items
    async getCallbackItems(items) {
        const itemsModel = items.map(async (data) => {
            const model = await this.widget_manager.get_model(data.callee);
            const item = Object.assign({}, data);
            item.model = model;
            return item;
        });
        return Promise.all(itemsModel);
    }
    getValueAtTime(traitName, time) {
        switch (traitName) {
            case 'ticks':
                return tone.Transport.getTicksAtTime(time);
            case 'seconds':
                return tone.Transport.getSecondsAtTime(time);
            default:
                return this.getValue(traitName);
        }
    }
    getValue(traitName) {
        switch (traitName) {
            case 'ticks':
                return tone.Transport.ticks;
            case 'seconds':
                return tone.Transport.seconds;
            case 'progress':
                return tone.Transport.progress;
            case 'position':
                return tone.Transport.position;
            case 'state':
                return tone.Transport.state;
            default:
                throw new Error('unsupported trait name ' + traitName);
        }
    }
    getToneCallback(items) {
        const callback = (time) => {
            items.forEach((item) => {
                const args = {};
                for (const [k, v] of Object.entries(item.args)) {
                    if (v.eval) {
                        args[k] = { value: eval(v.value), eval: true };
                    }
                    else {
                        args[k] = { value: v.value, eval: false };
                    }
                }
                const argsArray = (0, utils_1.normalizeArguments)(args, item.arg_keys);
                item.model.node[item.method](...argsArray);
            });
        };
        return callback;
    }
    async schedule(command) {
        const items = await this.getCallbackItems(command.items);
        const callback = this.getToneCallback(items);
        let event_id = 0;
        if (command.op === '') {
            event_id = tone.Transport.schedule(callback, command.time);
        }
        else if (command.op === 'repeat') {
            const duration = command.duration ? command.duration : Infinity;
            event_id = tone.Transport.scheduleRepeat(callback, command.interval, command.start_time, duration);
        }
        else if (command.op === 'once') {
            event_id = tone.Transport.scheduleOnce(callback, command.time);
        }
        this.py2jsEventID[command.id] = event_id;
    }
    syncPosition() {
        this.set('position', tone.Transport.position, { silent: true });
        this.set('seconds', tone.Transport.seconds, { silent: true });
        this.set('progress', tone.Transport.progress, { silent: true });
        this.set('ticks', tone.Transport.ticks, { silent: true });
        this.save_changes();
    }
    syncState() {
        this.set('state', tone.Transport.state, { silent: true });
        this.save_changes();
    }
    play(command) {
        const argsArray = (0, utils_1.normalizeArguments)(command.args, command.arg_keys);
        tone.Transport[command.method](...argsArray);
        this.syncPosition();
    }
    clearEvent(pyEventID) {
        tone.Transport.clear(this.py2jsEventID[pyEventID]);
        delete this.py2jsEventID[pyEventID];
    }
    async syncSignal(command) {
        const signalModel = (await this.widget_manager.get_model(command.signal));
        if (command.op === 'sync') {
            tone.Transport.syncSignal(signalModel.node, command.ratio);
            signalModel.set('value', 0, { silent: true });
            signalModel.save_changes();
        }
        else if (command.op === 'unsync') {
            tone.Transport.unsyncSignal(signalModel.node);
        }
    }
    handleMsg(command, _buffers) {
        if (command.event === 'schedule') {
            this.schedule(command);
        }
        else if (command.event === 'play') {
            this.play(command);
        }
        else if (command.event === 'clear') {
            this.clearEvent(command.id);
        }
        else if (command.event === 'cancel') {
            tone.Transport.cancel(command.after);
        }
        else if (command.event === 'sync_signal') {
            this.syncSignal(command);
        }
    }
    initEventListeners() {
        this.on('change:time_signature', () => {
            tone.Transport.timeSignature = this.get('time_signature');
        });
        this.on('change:loop_start', () => {
            tone.Transport.loopStart = this.get('loop_start');
        });
        this.on('change:loop_end', () => {
            tone.Transport.loopEnd = this.get('loop_end');
        });
        this.on('change:loop', () => {
            tone.Transport.loop = this.get('loop');
        });
        this.on('change:swing', () => {
            tone.Transport.swing = this.get('swing');
        });
        this.on('change:swing_subdivision', () => {
            tone.Transport.swingSubdivision = this.get('swing_subdivision');
        });
        this.on('change:position', () => {
            tone.Transport.position = this.get('position');
            this.syncPosition();
        });
        this.on('change:seconds', () => {
            tone.Transport.seconds = this.get('seconds');
            this.syncPosition();
        });
        this.on('change:ticks', () => {
            tone.Transport.position = this.get('ticks');
            this.syncPosition();
        });
        this.on('msg:custom', this.handleMsg, this);
        tone.Transport.on('start', () => {
            this.syncState();
        });
        tone.Transport.on('pause', () => {
            this.syncState();
            this.syncPosition();
        });
        tone.Transport.on('stop', () => {
            this.syncState();
            this.syncPosition();
        });
        // TODO: this will be available in the next Tone.js release
        // tone.Transport.on('ticks', () => {
        //   this.syncPosition();
        // });
    }
}
exports.TransportModel = TransportModel;
TransportModel.serializers = Object.assign(Object.assign({}, widget_base_1.ToneObjectModel.serializers), { _bpm: { deserialize: base_1.unpack_models } });
TransportModel.model_name = 'TransportModel';
//# sourceMappingURL=widget_transport.js.map